;nyquist plug-in
;version 4
;type generate
;name "Recording  Time Labels..."
;debugbutton true
;preview linear
;action "Generating Labels..."
;author "Steve Daulton"
;copyright "Released under terms of the GNU General Public License version 2"


;control m-interval "Label every" int "minutes past the hour" 5 0 59
;control s-interval "Label every" int "seconds past the hour" 0 0 60


;;; Error strings

(setf format-error
  (format nil "Error.~%~
          'Track Name' is incorrectly formatted.~%~%~
          The track should be recorded with~%~
          'Custom Track Name' + 'System Time'~%~
          options enabled in Preferences.~%~
          This will give a track name that ends~%~
          with: hh-mm-ss."))

;; We can't actually tell if a track was selected (Audacity 2.2.0) because Generate effects
;; automatically create an audio track if none selected. We can however check if an audio
;; track is empty.
(setf empty-track-error
  (format nil "Error,~%~
          Audio track is empty. ~%~%~
          An audio track must be selected, that has the~%~
          start time in the form 'hh-mm-ss' appended to the~%~
          track name.~%~
          Enabling the 'Custom Track Name' and 'System Time'~%~
          options in the Recording Preferences, will
          automatically label newly recorded tracks in a~%~
          compatible format."))

(defun too-many-labels (dur interval number)
  (format nil "Error.~%~
          Too many labels.~%~%~
          Maximum number of labels allowed between~%~
          the start and end of the track is 1000.~%~
          Track duration is ~a.~%~
          Labels every ~a would produce~%~
          ~a labels."
          (seconds-to-time (round dur) t t)
          (seconds-to-time interval t t)
          number))


;;; Time / Strng formatting helpers

(defmacro time-to-seconds (h m s)
"Convert 'h m s' to time in seconds"
  `(+ (* ,h 3600) (* ,m 60) ,s))


(defmacro seconds-to-time (seconds &optional short-format units)
"Convert seconds to hh-mm-ss (default) or short version"
  `(let ((ss (rem ,seconds 3600 60))
         (mm (rem (truncate (/ ,seconds 60)) 60))
         (hh (truncate (/ ,seconds 3600))))
    (format-time-string hh mm ss ,short-format ,units)))

(defun format-time-string (h m s &optional short units)
"Format time string."
  (let ((hh (pad-zeros h))
        (mm (pad-zeros m))
        (ss (pad-zeros s)))
    (cond
      (units
        (setf hh (strcat hh "h "))
        (setf mm (strcat mm "m "))
        (if (and (> (+ h m) 0)
                 (> s 9))
            (setf ss (strcat ss "s"))
            (setf ss (format nil "~as" s))))
      (t
        (setf hh (strcat hh "-"))
        (setf mm (strcat mm "-"))))
    (cond
      ((or (not short) (> h 0)) (strcat hh mm ss))
      ((> m 0) (strcat mm ss))
      (t ss))))

(defun pad-zeros (nn)
"Convert number to minimum 2 character string"
  (if (and (>= nn 0)(< nn 10))
      (format nil "0~a" nn)
      (format nil "~a" nn)))

(defmacro string-int (str)
"Convert a string integer to integer"
  `(do ((i 0 (1+ i))
        (num 0))
       ((= i (length ,str)) num)
     (setf num (+ (char-digit (char ,str i))
                  (* num 10)))))

(defmacro char-digit (ch)
"Converts an integer character to a digit"
  `(if (not (digit-char-p ,ch))
       (throw 'err format-error)
       (- (char-int ,ch) 48)))

(defun time-from-string (str)
"Extracts the time in seconds from a string 'somename-hh-mm-ss'"
  (if (< (length str) 8)
      (throw 'err format-error))
  (setf timestring (subseq str (- (length str) 8)))
  (let ((hh (subseq timestring 0 2))
        (mm (subseq timestring 3 5))
        (ss (subseq timestring 6)))
    (psetq hh (string-int hh)
           mm (string-int mm)
           ss (string-int ss))
    ;(print track-name)(print hh)(print mm)(print ss)
    ;(print (setf seconds (time-to-seconds hh mm ss)))
    ;(print (seconds-to-time seconds))
    (time-to-seconds hh mm ss)))

;;; Label functions

(defun one-label (time text)
"Make a single label"
  (setf offset (- (get '*track* 'start-time)
                  (get '*selection* 'start)))
  (list (+ time offset) text))


(defun make-labels (track-duration interval)
"Generate list of labels"
  (setf labels ())
  (let* ((range (track-time-range track-duration)) ;recording start/end times (seconds)
         (start (first range))
         (end (second range)))
    ;; Add label at start of track
    (push (one-label 0 (seconds-to-time start)) labels)
    ;; First label needs to be exact multiple of interval past the hour
    (setf label1-time (first-label-time start interval))
    (do* ((time label1-time (+ time interval)))
         ((>= time end))
      (setf track-time (- time start))
      (push (one-label track-time (seconds-to-time time)) labels))
    ;; Add label at end of track
    (push (one-label track-duration (seconds-to-time (round end))) labels)))


(defun number-of-labels (dur interval)
  (if (<= interval 0) ;should never happen
      (throw 'err "Divide by zero error."))
  (truncate (/ dur interval)))


;;; Main function  

(defun process ()
  (setf track-name (get '*track* 'name))
  (setf interval
    (if (> (+ s-interval m-interval) 0)
        (+ (* m-interval 60) s-interval)
        3600))
  (let*((track-duration (- (get '*track* 'end-time)
                           (get '*track* 'start-time)))
        (num (number-of-labels track-duration interval)))
    (if (> num 1000)
        (throw 'err (too-many-labels track-duration interval num)))
    
    (make-labels track-duration interval)))


(defun track-time-range (duration)
"Returns a list of recording start and 
  end time of the track in seconds"
  (let* ((track-start-time (time-from-string track-name))
         (track-end-time (+ track-start-time duration)))
    (list track-start-time track-end-time)))


(defun first-label-time (record-start-time interval)
"Return the recording time for the first label after the start of
  the recording (the first multiple of interval past the hour)."
  (let* ((mins-after-hour (rem record-start-time 3600))
         (hour (- record-start-time mins-after-hour))
         (intervals-after-hour (truncate (/ mins-after-hour interval))))
    (+ hour (* (1+ intervals-after-hour) interval))))

;(catch 'err (time-from-string track-name))
(if (= (get '*track* 'start-time) (get '*track* 'end-time))
    empty-track-error
    (catch 'err (process)))
